/*
 *------------------------------------------------------------------
 *
 * threads.h
 * Nov 98
 * 304-427A
 * Sorin Lerner (based on original implementation from
 * prof. Cooperstock)
 *
 * Function prototypes and public data for the threads library.
 *
 *------------------------------------------------------------------
 */

#ifndef _THREADS_H_
#define _THREADS_H_

/*
 * Error constants. I've changed the name of these, so that they all start
 * with "E_". It is usually a good idea to "scope" your constants this
 * way, because it removes ambiguities. For example, a constant like
 * NOT_INITIALIZED could be an error code, or a status constant. You can
 * disambiguate by using E_NOT_INITIALIZED if it's an error, or
 * S_NOT_INITIALIZED if it's a status.
 * Renaming the constans doesn't affect the solution.
 *
 * Note that E_NO_MORE_TD and E_INVALID_TID have the same
 * value since they both correspond to the ERROR constant given in the
 * assignment.
 *
 */

#define OK 		 0
#define E_STACK_TOO_SMALL -100
#define E_NO_MORE_TD	-101
#define E_INVALID_TID	-101
#define E_INVALID_PRI	-102
#define E_NOT_BLOCKED	-103
#define E_NOT_INITIALIZED -104
#define E_OUT_OF_MEM	-105
#define E_ALREADY_INITIALIZED -106
#define E_LAST_READY -107
#define E_NOT_IMPL -108

/*
 * Min and max values for priorities, stack size and number of threads
 */

#define MIN_PRIORITY	99
#define MAX_PRIORITY	-99
#define MAX_THREAD	32
#define MIN_STACK_SIZE 	(8 * 1024)


/*
 * Types used to interact with clients of the library. Just one...
 */

typedef int ThreadId;


/*
 * Functions in the library
 */


/* intitialize */

int ThreadsInit();


/* to force use of the scheduler */

int ThreadsUseScheduler();


/* thread manipulation */

ThreadId ThreadCreate(unsigned pc,unsigned stack_size,int priority); 
int ThreadDestroy(ThreadId id);
int ThreadYield();
int ThreadBlock();
int ThreadWakeUp(ThreadId id);
int ThreadChangePriority(ThreadId id,int new_priority);

/* idle function */

void Idle();

#endif /* #ifndef _THREADS_H_ */
